@@ -27,10 +27,12 @@ use anchor_lang::solana_program;
 use vipers::prelude::*;
 
 mod events;
+mod instructions;
 mod state;
 mod validators;
 
 pub use events::*;
+pub use instructions::*;
 pub use state::*;
 
 /// Number of seconds in a day.
@@ -386,6 +388,38 @@ pub mod smart_wallet {
 
         Ok(())
     }
+
+    #[access_control(ctx.accounts.validate())]
+    pub fn init_ix_buffer(ctx: Context<InitBuffer>, eta: i64) -> Result<()> {
+        instructions::buffer_init::handler(ctx, eta)
+    }
+
+    #[access_control(ctx.accounts.validate())]
+    pub fn close_ix_buffer(ctx: Context<CloseBuffer>) -> Result<()> {
+        instructions::buffer_close::handler(ctx)
+    }
+
+    #[access_control(ctx.accounts.validate())]
+    pub fn execute_buffer_bundle<'info>(
+        ctx: Context<'_, '_, '_, 'info, ExecuteBufferBundle<'info>>,
+        bundle_index: u8,
+    ) -> Result<()> {
+        instructions::buffer_execute_bundle::handler(ctx, bundle_index)
+    }
+
+    #[access_control(ctx.accounts.validate())]
+    pub fn append_buffer_ix(
+        ctx: Context<AppendBufferIX>,
+        bundle_index: u8,
+        ix: TXInstruction,
+    ) -> Result<()> {
+        instructions::buffer_append_ix::handler(ctx, bundle_index, ix)
+    }
+
+    #[access_control(ctx.accounts.validate())]
+    pub fn finalize_buffer(ctx: Context<FinalizeBuffer>) -> Result<()> {
+        instructions::buffer_finalize::handler(ctx)
+    }
 }
 
 /// Accounts for [smart_wallet::create_smart_wallet].
@@ -486,15 +520,6 @@ pub struct OwnerInvokeInstruction<'info> {
     pub owner: Signer<'info>,
 }
 
-/// Accounts for [smart_wallet::owner_invoke_instruction].
-#[derive(Accounts)]
-pub struct OwnerInvokeInstructionV2<'info> {
-    /// The [SmartWallet].
-    pub smart_wallet: Account<'info, SmartWallet>,
-    /// An owner of the [SmartWallet].
-    pub owner: Signer<'info>,
-}
-
 /// Accounts for [smart_wallet::create_subaccount_info].
 #[derive(Accounts)]
 #[instruction(bump: u8, subaccount: Pubkey)]
@@ -559,4 +584,12 @@ pub enum ErrorCode {
     OwnerSetChanged,
     #[msg("Subaccount does not belong to smart wallet.")]
     SubaccountOwnerMismatch,
+    #[msg("Buffer already finalized.")]
+    BufferFinalized,
+    #[msg("Buffer bundle not found.")]
+    BufferBundleNotFound,
+    #[msg("Buffer has not been finalized.")]
+    BufferBundleNotFinalized,
+    #[msg("Buffer bundle has already been executed.")]
+    BufferBundleExecuted,
 }
